<?php

namespace Oro\Bundle\PricingBundle\Entity\Repository;

use Doctrine\ORM\Query;
use Doctrine\ORM\Query\Expr\Join;
use Doctrine\ORM\QueryBuilder;
use Oro\Bundle\PricingBundle\Entity\CombinedPriceList;
use Oro\Bundle\PricingBundle\Entity\CombinedPriceListToCustomer;
use Oro\Bundle\PricingBundle\Entity\CombinedPriceListToCustomerGroup;
use Oro\Bundle\PricingBundle\Entity\CombinedPriceListToWebsite;
use Oro\Bundle\PricingBundle\Entity\CombinedProductPrice;
use Oro\Bundle\PricingBundle\Entity\PriceList;
use Oro\Bundle\PricingBundle\ORM\InsertFromSelectShardQueryExecutor;
use Oro\Bundle\PricingBundle\ORM\Walker\PriceShardWalker;
use Oro\Bundle\PricingBundle\Sharding\ShardManager;
use Oro\Bundle\ProductBundle\Entity\Product;

class CombinedProductPriceRepository extends BaseProductPriceRepository
{
    const BATCH_SIZE = 100000;

    /**
     * @param InsertFromSelectShardQueryExecutor $insertFromSelectQueryExecutor
     * @param CombinedPriceList $combinedPriceList
     * @param PriceList $priceList
     * @param boolean $mergeAllowed
     * @param Product|null $product
     */
    public function insertPricesByPriceList(
        InsertFromSelectShardQueryExecutor $insertFromSelectQueryExecutor,
        CombinedPriceList $combinedPriceList,
        PriceList $priceList,
        $mergeAllowed,
        Product $product = null
    ) {
        $qb = $this->getEntityManager()
            ->getRepository('OroPricingBundle:ProductPrice')
            ->createQueryBuilder('pp');

        $qb
            ->select(
                'IDENTITY(pp.product)',
                'IDENTITY(pp.unit)',
                (string)$qb->expr()->literal($combinedPriceList->getId()),
                'pp.productSku',
                'pp.quantity',
                'pp.value',
                'pp.currency',
                sprintf('CAST(%d as boolean)', (int)$mergeAllowed)
            )
            ->where($qb->expr()->eq('pp.priceList', ':currentPriceList'))
            ->andWhere($qb->expr()->between('pp.product', ':product_min', ':product_max'))
            ->groupBy('pp.id')
            ->setParameter('currentPriceList', $priceList);
        $this->addUniquePriceCondition($qb, $combinedPriceList, $mergeAllowed);
        $minProductId = null;
        $maxProductId = null;
        if ($product) {
            $minProductId = $product->getId();
            $maxProductId = $product->getId();
        } else {
            $productPriceQb = $this->getEntityManager()
                ->createQueryBuilder();
            $productPriceQb->select('MIN(IDENTITY(ptp.product))')
                ->from('OroPricingBundle:PriceListToProduct', 'ptp')
                ->where('ptp.priceList = :priceList')
                ->setParameter('priceList', $priceList);

            $minProductId = $productPriceQb->getQuery()->getSingleScalarResult();
            $maxProductId = $productPriceQb->select('MAX(IDENTITY(ptp.product))')->getQuery()->getSingleScalarResult();
        }
        while ($minProductId <= $maxProductId) {
            $currentMax = $minProductId + self::BATCH_SIZE;
            if ($currentMax > $maxProductId) {
                $currentMax = $maxProductId;
            }
            $qb->setParameter('product_min', $minProductId)
                ->setParameter('product_max', $currentMax);

            $insertFromSelectQueryExecutor->execute(
                'OroPricingBundle:CombinedProductPrice',
                [
                    'product',
                    'unit',
                    'priceList',
                    'productSku',
                    'quantity',
                    'value',
                    'currency',
                    'mergeAllowed'
                ],
                $qb
            );
            // +1 because between operator includes boundary values
            $minProductId = $currentMax + 1;
        }
    }

    /**
     * @param CombinedPriceList $combinedPriceList
     * @param Product|null $product
     */
    public function deleteCombinedPrices(CombinedPriceList $combinedPriceList, Product $product = null)
    {
        $qb = $this->getEntityManager()->createQueryBuilder();
        $qb->delete($this->getEntityName(), 'combinedPrice')
            ->where($qb->expr()->eq('combinedPrice.priceList', ':combinedPriceList'))
            ->setParameter('combinedPriceList', $combinedPriceList);

        if ($product) {
            $qb->andWhere($qb->expr()->eq('combinedPrice.product', ':product'))
                ->setParameter('product', $product);
        }

        $qb->getQuery()->execute();
    }

    /**
     * @param CombinedPriceList $priceList
     * @param array $productIds
     * @param null|string $currency
     * @return CombinedProductPrice[]
     */
    public function getPricesForProductsByPriceList(CombinedPriceList $priceList, array $productIds, $currency = null)
    {
        if (count($productIds) === 0) {
            return [];
        }

        $qb = $this->createQueryBuilder('cpp');

        $qb->select('cpp')
            ->where($qb->expr()->eq('cpp.priceList', ':priceList'))
            ->andWhere($qb->expr()->in('cpp.product', ':productIds'))
            ->setParameters([
                'priceList' => $priceList,
                'productIds' => $productIds
            ]);

        if ($currency) {
            $qb->andWhere($qb->expr()->eq('cpp.currency', ':currency'))
                ->setParameter('currency', $currency);
        }

        return $qb->getQuery()->getResult();
    }

    /**
     * @param QueryBuilder $qb
     * @param CombinedPriceList $combinedPriceList
     * @param boolean $mergeAllowed
     */
    protected function addUniquePriceCondition(
        QueryBuilder $qb,
        CombinedPriceList $combinedPriceList,
        $mergeAllowed
    ) {
        $additionalProductCondition = $qb->expr()->between('cpp.product', ':product_min', ':product_max');
        if ($mergeAllowed) {
            $qb->leftJoin(
                'OroPricingBundle:CombinedProductPrice',
                'cpp',
                Join::WITH,
                $qb->expr()->andX(
                    $qb->expr()->eq('cpp.priceList', ':combinedPriceList'),
                    $qb->expr()->eq('pp.product', 'cpp.product'),
                    $qb->expr()->eq('cpp.mergeAllowed', ':mergeAllowed'),
                    $additionalProductCondition
                )
            )
            ->setParameter('mergeAllowed', false);

            $additionalProductCondition = $qb->expr()->between('cpp2.product', ':product_min', ':product_max');
            $qb->leftJoin(
                'OroPricingBundle:CombinedProductPrice',
                'cpp2',
                Join::WITH,
                $qb->expr()->andX(
                    $qb->expr()->eq('cpp2.priceList', ':combinedPriceList'),
                    $qb->expr()->eq('pp.product', 'cpp2.product'),
                    $qb->expr()->eq('pp.unit', 'cpp2.unit'),
                    $qb->expr()->eq('pp.quantity', 'cpp2.quantity'),
                    $qb->expr()->eq('pp.currency', 'cpp2.currency'),
                    $additionalProductCondition
                )
            )
            ->andWhere($qb->expr()->isNull('cpp2.id'));
        } else {
            $qb->leftJoin(
                'OroPricingBundle:CombinedProductPrice',
                'cpp',
                Join::WITH,
                $qb->expr()->andX(
                    $qb->expr()->eq('cpp.priceList', ':combinedPriceList'),
                    $qb->expr()->eq('pp.product', 'cpp.product'),
                    $additionalProductCondition
                )
            );
        }

        $qb->andWhere($qb->expr()->isNull('cpp.id'))
            ->setParameter('combinedPriceList', $combinedPriceList->getId());
    }

    /**
     * {@inheritdoc}
     */
    public function findByPriceListIdAndProductIds(
        ShardManager $shardManager,
        $priceListId,
        array $productIds,
        $getTierPrices = true,
        $currency = null,
        $productUnitCode = null,
        array $orderBy = ['unit' => 'ASC', 'quantity' => 'ASC']
    ) {
        if (!$productIds) {
            return [];
        }

        $qb = $this->getFindByPriceListIdAndProductIdsQueryBuilder(
            $priceListId,
            $productIds,
            $getTierPrices,
            $currency,
            $productUnitCode,
            $orderBy
        );
        $qb
            ->addSelect('product', 'unitPrecisions', 'unit')
            ->leftJoin('price.product', 'product')
            ->leftJoin('product.unitPrecisions', 'unitPrecisions')
            ->leftJoin('unitPrecisions.unit', 'unit');

        return $qb->getQuery()->getResult();
    }

    /**
     * @param integer $websiteId
     * @param Product[] $products
     * @param CombinedPriceList $configCpl
     * @return array
     */
    public function findMinByWebsiteForFilter($websiteId, array $products, $configCpl)
    {
        $qb = $this->getQbForMinimalPrices($websiteId, $products, $configCpl);
        $qb->select(
            'IDENTITY(mp.product) as product',
            'MIN(mp.value) as value',
            'mp.currency',
            'IDENTITY(mp.priceList) as cpl',
            'IDENTITY(mp.unit) as unit'
        );
        $qb->groupBy('mp.product, mp.priceList, mp.currency, mp.unit');

        return $qb->getQuery()->getArrayResult();
    }

    /**
     * @param integer $websiteId
     * @param Product[] $products
     * @param CombinedPriceList $configCpl
     * @return array
     */
    public function findMinByWebsiteForSort($websiteId, array $products, $configCpl)
    {
        $qb = $this->getQbForMinimalPrices($websiteId, $products, $configCpl);
        $qb->select(
            'IDENTITY(mp.product) as product',
            'MIN(mp.value) as value',
            'mp.currency',
            'IDENTITY(mp.priceList) as cpl'
        );
        $qb->groupBy('mp.product, mp.priceList, mp.currency');

        return $qb->getQuery()->getArrayResult();
    }

    /**
     * @param ShardManager $shardManager
     * @param InsertFromSelectShardQueryExecutor $insertFromSelectQueryExecutor
     * @param CombinedPriceList $combinedPriceList
     * @param PriceList $priceList
     * @param Product|null $product
     */
    public function insertMinimalPricesByPriceList(
        ShardManager $shardManager,
        InsertFromSelectShardQueryExecutor $insertFromSelectQueryExecutor,
        CombinedPriceList $combinedPriceList,
        PriceList $priceList,
        Product $product = null
    ) {
        //remove prices that are greater of prices from current PriceList
        $this->deleteInvalidPricesForMinimalStrategy($shardManager, $combinedPriceList, $priceList, $product);

        //insert all prices to free slots
        $this->insertPricesByPriceList(
            $insertFromSelectQueryExecutor,
            $combinedPriceList,
            $priceList,
            true,
            $product
        );
    }

    /**
     * @param $websiteId
     * @param array $products
     * @param $configCpl
     * @return QueryBuilder
     */
    protected function getQbForMinimalPrices($websiteId, array $products, $configCpl)
    {
        $qb = $this->createQueryBuilder('mp');
        $qb->where('mp.product in (:products)')
            ->setParameter('products', $products);
        $expr = $qb->expr()->orX(
            $qb->expr()->orX(
                $qb->expr()->exists(
                    $this->getEntityManager()
                        ->createQueryBuilder()
                        ->from(CombinedPriceListToWebsite::class, 'cpl_w')
                        ->select('cpl_w.id')
                        ->where('cpl_w.website = :websiteId')
                        ->andWhere('cpl_w.priceList = mp.priceList')
                        ->getDQL()
                ),
                $qb->expr()->exists(
                    $this->getEntityManager()
                        ->createQueryBuilder()
                        ->from(CombinedPriceListToCustomer::class, 'cpl_a')
                        ->select('cpl_a.id')
                        ->where('cpl_a.website = :websiteId')
                        ->andWhere('cpl_a.priceList = mp.priceList')
                        ->getDQL()
                ),
                $qb->expr()->exists(
                    $this->getEntityManager()
                        ->createQueryBuilder()
                        ->from(CombinedPriceListToCustomerGroup::class, 'cpl_ag')
                        ->select('cpl_ag.id')
                        ->where('cpl_ag.website = :websiteId')
                        ->andWhere('cpl_ag.priceList = mp.priceList')
                        ->getDQL()
                )
            )
        );
        if ($configCpl) {
            $expr->add('mp.priceList = :conf_cpl');
            $qb->setParameter('conf_cpl', $configCpl);
        }
        $qb->andWhere($expr)
            ->setParameter('websiteId', $websiteId);

        return $qb;
    }

    /**
     * @param ShardManager $shardManager
     * @param CombinedPriceList $combinedPriceList
     * @param PriceList $priceList
     * @param Product|null $product
     */
    protected function deleteInvalidPricesForMinimalStrategy(
        ShardManager $shardManager,
        CombinedPriceList $combinedPriceList,
        PriceList $priceList,
        Product $product = null
    ) {
        $invalidPricesQb = $this->createQueryBuilder('cpp');
        $invalidPricesQb->select('DISTINCT cpp.id')
            ->where('cpp.priceList = :cpl')
            ->andWhere($invalidPricesQb->expr()->between('cpp.product', ':product_min', ':product_max'))
            ->setParameter('cpl', $combinedPriceList);
        $invalidPricesQb->join(
            'OroPricingBundle:ProductPrice',
            'pp',
            Join::WITH,
            $invalidPricesQb->expr()->andX(
                $invalidPricesQb->expr()->eq('pp.priceList', ':priceList'),
                $invalidPricesQb->expr()->eq('pp.product', 'cpp.product'),
                $invalidPricesQb->expr()->eq('pp.unit', 'cpp.unit'),
                $invalidPricesQb->expr()->eq('pp.quantity', 'cpp.quantity'),
                $invalidPricesQb->expr()->eq('pp.currency', 'cpp.currency'),
                $invalidPricesQb->expr()->lt('pp.value', 'cpp.value')
            )
        );
        $invalidPricesQb->setParameter('priceList', $priceList);
        $minProductId = null;
        $maxProductId = null;
        if ($product) {
            $minProductId = $product->getId();
            $maxProductId = $product->getId();
        } else {
            $productPriceQb = $this->getEntityManager()
                ->createQueryBuilder();
            $productPriceQb->select('MIN(IDENTITY(ptp.product))')
                ->from('OroPricingBundle:PriceListToProduct', 'ptp')
                ->where('ptp.priceList = :priceList')
                ->setParameter('priceList', $priceList);

            $minProductId = $productPriceQb->getQuery()->getSingleScalarResult();
            $maxProductId = $productPriceQb->select('MAX(IDENTITY(ptp.product))')->getQuery()->getSingleScalarResult();
        }
        while ($minProductId <= $maxProductId) {
            $currentMax = $minProductId + self::BATCH_SIZE;
            if ($currentMax > $maxProductId) {
                $currentMax = $maxProductId;
            }
            $invalidPricesQb->setParameter('product_min', $minProductId)
                ->setParameter('product_max', $currentMax);
            $query = $invalidPricesQb->getQuery();
            $query->setHint('priceList', $priceList->getId());
            $query->setHint(PriceShardWalker::ORO_PRICING_SHARD_MANAGER, $shardManager);
            $query->setHint(Query::HINT_CUSTOM_OUTPUT_WALKER, PriceShardWalker::class);
            $ids = $query->getScalarResult();
            $this->deletePricesByIds($ids);
            // +1 because between operator includes boundary values
            $minProductId = $currentMax + 1;
        }
    }

    /**
     * @param array $prices
     */
    protected function deletePricesByIds(array $prices)
    {
        if (empty($prices)) {
            return;
        }
        $this->_em->createQueryBuilder()
           ->delete($this->_entityName, 'prices')
           ->where('prices.id IN(:prices)')
           ->setParameter('prices', $prices)
           ->getQuery()
           ->execute();
    }
}
